﻿using UnityEngine;
using System.Collections;

public enum DataportReturn
{
	DATAPORT_SUCCESS = 0,
	DATAPORT_ERROR_NAMECLASH,
	DATAPORT_ERROR_NOMORESLOTS,
	DATAPORT_WARINING_BEINGREFERENCED,
	DATAPORT_ERROR_ENTRYNOTFOUND,
	DATAPORT_ERROR_NOTATTACHED,
	DATAPORT_ERROR_WRONGTYPE,
	DATAPORT_ERROR_REFCOUNTZERO
};

public class DataportManager 
{
	private const int 		maxNumberOfDataports = 200;
	private object[]		dataports;
	private int[]	 		dataportHashes;
	private int[]	 		dataportRefCounter;
	private int 			numDataports;

	static readonly DataportManager _instance = new DataportManager();

	public static DataportManager Instance
	{
		get{return _instance;}
	}

	public DataportManager()
	{
		dataports 			= new object[maxNumberOfDataports];
		dataportHashes 		= new int[maxNumberOfDataports];
		dataportRefCounter 	= new int[maxNumberOfDataports];
		numDataports 		= 0;
	}

	static int GetStringHash(string str)
	{
		int result = 0;

		foreach(int ascii in str)
		{
			unchecked
			{
				result += ascii;
			}
		}

		return result;
	}

	public DataportReturn Add(string id, object obj)
	{
		int ID = GetStringHash(id);
		if(numDataports < maxNumberOfDataports)
		{
			for(int i = 0; i < numDataports; i++)
			{
				if(dataports[i] == obj)
				{
					return DataportReturn.DATAPORT_ERROR_NAMECLASH;
				}
			}

			dataports[numDataports] = obj;
			dataportHashes[numDataports] = ID;
			dataportRefCounter[numDataports] = 0;

			numDataports++;

			return DataportReturn.DATAPORT_SUCCESS;
		}
		else
		{
			return DataportReturn.DATAPORT_ERROR_NOMORESLOTS;
		}
	}

	public DataportReturn Remove(object obj)
	{
		for(int i = 0; i < numDataports; i++)
		{
			if(dataports[i] == obj)
			{
				numDataports--;
				dataports[i] = dataports[numDataports];
				dataportHashes[i] = dataportHashes[numDataports];
				dataportRefCounter[i] = dataportRefCounter[numDataports];

				dataports[numDataports] = null;
				dataportHashes[numDataports] = 0;

				if(dataportRefCounter[numDataports] > 0)
				{
					return DataportReturn.DATAPORT_WARINING_BEINGREFERENCED;
				}

				return DataportReturn.DATAPORT_SUCCESS;
			}
		}
		return DataportReturn.DATAPORT_ERROR_ENTRYNOTFOUND;
	}

	public object Attach(string id)
	{
		int ID = GetStringHash(id);

		for(int i = 0; i < numDataports; i++)
		{
			if(dataportHashes[i] == ID)
			{
				dataportRefCounter[i]++;
				return dataports[i];
			}
		}
		return null;
	}

	public DataportReturn Detach(object obj)
	{
		for(int i = 0; i < numDataports; i++)
		{
			if(dataports[i] == obj)
			{
				if(dataportRefCounter[i] > 0)
				{
					dataportRefCounter[i]--;
					return DataportReturn.DATAPORT_SUCCESS;
				}
				else
				{
 					return DataportReturn.DATAPORT_ERROR_NOTATTACHED;
				}
			}
		}
		return DataportReturn.DATAPORT_ERROR_ENTRYNOTFOUND;
	}
}
